==> reverie_dreams-0.4.7%2B1.21.7%2Bbeta-sources.jar: cc/thonly/polymer/block/GensokyoAltarImpl.java <==
package cc.thonly.polymer.block;

import cc.thonly.reverie_dreams.block.GensokyoAltarBlock;
import cc.thonly.reverie_dreams.block.entity.GensokyoAltarBlockEntity;
import eu.pb4.factorytools.api.block.FactoryBlock;
import eu.pb4.factorytools.api.virtualentity.BlockModel;
import eu.pb4.factorytools.api.virtualentity.ItemDisplayElementUtil;
import eu.pb4.polymer.blocks.api.PolymerTexturedBlock;
import eu.pb4.polymer.virtualentity.api.ElementHolder;
import eu.pb4.polymer.virtualentity.api.attachment.BlockBoundAttachment;
import eu.pb4.polymer.virtualentity.api.attachment.HolderAttachment;
import eu.pb4.polymer.virtualentity.api.elements.ItemDisplayElement;
import org.jetbrains.annotations.Nullable;
import org.joml.Vector3f;
import xyz.nucleoid.packettweaker.PacketContext;

import java.util.HashMap;
import java.util.Map;
import net.minecraft.class_1277;
import net.minecraft.class_1799;
import net.minecraft.class_2246;
import net.minecraft.class_2338;
import net.minecraft.class_243;
import net.minecraft.class_2680;
import net.minecraft.class_3218;

public class GensokyoAltarImpl implements FactoryBlock, PolymerTexturedBlock {
    public static final Map<class_2680, Model> STATE_TO_MODEL = new HashMap<>();

    @Override
    public class_2680 getPolymerBlockState(class_2680 state, PacketContext context) {
        return class_2246.field_10499.method_9564();
    }

    @Override
    public class_2680 getPolymerBreakEventBlockState(class_2680 state, PacketContext context) {
        return class_2246.field_10485.method_9564();
    }

    @Override
    public @Nullable ElementHolder createElementHolder(class_3218 world, class_2338 pos, class_2680 state) {
        Model altarModel = new Model(world, pos, state);
        STATE_TO_MODEL.put(state, altarModel);
        return altarModel;
    }

    public static class Model extends BlockModel {
        protected ItemDisplayElement main;
        public final ItemDisplayElement[] itemStackDisplay = new ItemDisplayElement[9];
        class_2680 state;
        class_2338 pos;
        class_3218 world;
        GensokyoAltarBlockEntity blockEntity;
        class_1277 inventory;

        public Model(class_3218 world, class_2338 pos, class_2680 state) {
            init(state);
            this.world = world;
            this.pos = pos;
            this.state = state;
        }

        public GensokyoAltarBlockEntity getBlockEntityFromWorld() {
            if (this.world == null) return null;
            return (GensokyoAltarBlockEntity) this.world.method_8321(this.pos);
        }

        public float angle = 0;

        public void update() {
            this.blockEntity = this.getBlockEntityFromWorld();
            if (this.blockEntity == null) return;
            this.inventory = this.blockEntity.getInventory();

            for (int i = 0; i < this.itemStackDisplay.length; i++) {
                class_1799 stack = this.inventory.method_5438(i);
                ItemDisplayElement element = this.itemStackDisplay[i];

                if (stack.method_7960()) {
                    if (element != null) {
                        this.removeElement(element);
                        this.itemStackDisplay[i] = null;
                    }
                    continue;
                }

                if (element == null || !class_1799.method_7973(stack, element.getItem())) {
                    if (element != null) {
                        this.removeElement(element);
                    }

                    ItemDisplayElement newElement = ItemDisplayElementUtil.createSimple(stack);
                    int[] offset = GensokyoAltarBlock.OFFSETS[i];
                    newElement.setScale(new Vector3f(i != 8 ? 0.6f : 0.5f));
                    newElement.setDisplaySize(0.5f, 0.5f);
                    newElement.setOffset(new class_243(offset[0], i != 8 ? 3 : 0.5, offset[1]));

                    this.addElement(newElement);
                    this.itemStackDisplay[i] = newElement;
                }
            }
        }

        public void init(class_2680 state) {
            main = ItemDisplayElementUtil.createSimple(state.method_26204().method_8389());
            main.setScale(new Vector3f(2f));
            addElement(main);
        }

        protected void updateItem(class_2680 state) {
            this.removeElement(main);
            init(state);
        }

        @Override
        public void notifyUpdate(HolderAttachment.UpdateType updateType) {
            if (updateType == BlockBoundAttachment.BLOCK_STATE_UPDATE) {
                updateItem(this.blockState());
                this.update();
            }
            super.notifyUpdate(updateType);
        }
    }
}

==> reverie_dreams-0.4.8%2B1.21.7%2Bbeta-sources.jar: cc/thonly/polymer/block/GensokyoAltarImpl.java <==
package cc.thonly.polymer.block;

import cc.thonly.reverie_dreams.block.GensokyoAltarBlock;
import cc.thonly.reverie_dreams.block.entity.GensokyoAltarBlockEntity;
import eu.pb4.factorytools.api.block.FactoryBlock;
import eu.pb4.factorytools.api.virtualentity.BlockModel;
import eu.pb4.factorytools.api.virtualentity.ItemDisplayElementUtil;
import eu.pb4.polymer.blocks.api.PolymerTexturedBlock;
import eu.pb4.polymer.virtualentity.api.ElementHolder;
import eu.pb4.polymer.virtualentity.api.attachment.BlockBoundAttachment;
import eu.pb4.polymer.virtualentity.api.attachment.HolderAttachment;
import eu.pb4.polymer.virtualentity.api.elements.ItemDisplayElement;
import org.jetbrains.annotations.Nullable;
import org.joml.Vector3f;
import xyz.nucleoid.packettweaker.PacketContext;

import java.util.HashMap;
import java.util.Map;
import net.minecraft.class_1277;
import net.minecraft.class_1799;
import net.minecraft.class_2246;
import net.minecraft.class_2338;
import net.minecraft.class_243;
import net.minecraft.class_2680;
import net.minecraft.class_3218;

public class GensokyoAltarImpl implements FactoryBlock, PolymerTexturedBlock {
    public static final Map<Long, Model> POS_TO_MODEL = new HashMap<>();

    @Override
    public class_2680 getPolymerBlockState(class_2680 state, PacketContext context) {
        return class_2246.field_10499.method_9564();
    }

    @Override
    public class_2680 getPolymerBreakEventBlockState(class_2680 state, PacketContext context) {
        return class_2246.field_10485.method_9564();
    }

    @Override
    public @Nullable ElementHolder createElementHolder(class_3218 world, class_2338 pos, class_2680 state) {
        Model altarModel = new Model(world, pos, state);
        POS_TO_MODEL.put(pos.method_10063(), altarModel);
        return altarModel;
    }

    public static class Model extends BlockModel {
        protected ItemDisplayElement main;
        public final ItemDisplayElement[] itemStackDisplay = new ItemDisplayElement[9];
        private class_2680 state;
        private class_2338 pos;
        private class_3218 world;
        private GensokyoAltarBlockEntity blockEntity;
        private class_1277 inventory;

        public Model(class_3218 world, class_2338 pos, class_2680 state) {
            init(state);
            this.world = world;
            this.pos = pos;
            this.state = state;
        }

        public GensokyoAltarBlockEntity getBlockEntityFromWorld() {
            if (this.world == null) {
                return null;
            }
            return (GensokyoAltarBlockEntity) this.world.method_8321(this.pos);
        }

        public float angle = 0;

        public void update() {
            this.blockEntity = this.getBlockEntityFromWorld();
            if (this.blockEntity == null) return;
            this.inventory = this.blockEntity.getInventory();

            for (int i = 0; i < this.itemStackDisplay.length; i++) {
                class_1799 stack = this.inventory.method_5438(i);
                ItemDisplayElement element = this.itemStackDisplay[i];

                if (stack.method_7960()) {
                    if (element != null) {
                        this.removeElement(element);
                        this.itemStackDisplay[i] = null;
                    }
                    continue;
                }

                if (element == null || !class_1799.method_7973(stack, element.getItem())) {
                    if (element != null) {
                        this.removeElement(element);
                    }

                    ItemDisplayElement newElement = ItemDisplayElementUtil.createSimple(stack);
                    int[] offset = GensokyoAltarBlock.OFFSETS[i];
                    newElement.setScale(new Vector3f(i != 8 ? 0.6f : 0.5f));
                    newElement.setDisplaySize(0.5f, 0.5f);
                    newElement.setOffset(new class_243(offset[0], i != 8 ? 3 : 0.5, offset[1]));

                    this.addElement(newElement);
                    this.itemStackDisplay[i] = newElement;
                }
            }
        }

        public void init(class_2680 state) {
            main = ItemDisplayElementUtil.createSimple(state.method_26204().method_8389());
            main.setScale(new Vector3f(2f));
            addElement(main);
        }

        protected void updateItem(class_2680 state) {
            this.removeElement(main);
            init(state);
        }

        @Override
        public void notifyUpdate(HolderAttachment.UpdateType updateType) {
            if (updateType == BlockBoundAttachment.BLOCK_STATE_UPDATE) {
                updateItem(this.blockState());
                this.update();
            }
            super.notifyUpdate(updateType);
        }
    }
}
